#include "PixelGeoModel/GeoPixelDisk.h"
#include "PixelGeoModel/GeoPixelSubDisk.h"
#include "PixelGeoModel/GeoPixelDiskSupports.h"
#include "PixelGeoModel/GeoPixelSiCrystal.h"

#include "GeoModelKernel/GeoTube.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
#include "GeoModelKernel/GeoIdentifierTag.h"
#include "GeoModelKernel/GeoFullPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"
#include "GeoModelKernel/GeoTransform.h"
#include "GeoModelKernel/GeoAlignableTransform.h"

#include "InDetReadoutGeometry/PixelDetectorManager.h"

GeoPixelDisk::GeoPixelDisk() {

  // Define the log volume in the constructor, so I do it only once.
  //

  //   
  // Dimensions from class methods.
  //
  double rmin = RMin();
  double rmax = RMax();
  double halflength = Thickness()/2.;
  GeoMaterial* air = mat_mgr->getMaterial("std::Air");
  const GeoTube* diskTube = new GeoTube(rmin,rmax,halflength);
  theDisk = new GeoLogVol("diskLog",diskTube,air);
}
GeoVPhysVol* GeoPixelDisk::Build( ) {
  //
  // Define the Sensor to be used here, so it will be the same for all the disk
  GeoPixelSiCrystal theSensor(false);
  GeoFullPhysVol* diskPhys = new GeoFullPhysVol(theDisk);
  //
  // Place the disk sectors (on both sides):
  //
  GeoPixelSubDisk psd(theSensor);
  double zpos = -gmt_mgr->PixelECSiDz1()/2.;
  double angle = 360.*deg/ (float) gmt_mgr->PixelECNSectors1();
  Hep3Vector pos(0.,0.,zpos);
  
  // Even modules
  gmt_mgr->SetEta(0);
  for (int ii = 0; ii <  gmt_mgr->PixelECNSectors1(); ii++) {
    gmt_mgr->SetPhi(ii);
    HepRotation rm;
    rm.rotateX(180.*deg );
    rm.rotateZ(ii*angle+angle/2.);
    GeoAlignableTransform* xform = new GeoAlignableTransform(HepTransform3D(rm,pos));
    GeoVPhysVol * modulePhys = psd.Build();
    GeoNameTag* tag = new GeoNameTag("DiskSector");
    diskPhys->add(tag);
    diskPhys->add(new GeoIdentifierTag(getPhiId() ) );
    diskPhys->add(xform);
    diskPhys->add(modulePhys);

    // Now store the xform by identifier:
    Identifier id = theSensor.getID();
    DDmgr->addAlignableTransform(0,id,xform,modulePhys);
  }

  // Odd modules
  gmt_mgr->SetEta(1);
  zpos = gmt_mgr->PixelECSiDz2()/2.;
  pos = Hep3Vector(0.,0.,zpos);
  for (int ii = 0; ii <  gmt_mgr->PixelECNSectors1(); ii++) {
    gmt_mgr->SetPhi(ii);
    HepRotation rm;
    rm.rotateZ((ii+1)*angle);
    GeoAlignableTransform* xform = new GeoAlignableTransform(HepTransform3D(rm,pos));
    GeoVPhysVol * modulePhys = psd.Build();
    GeoNameTag* tag = new GeoNameTag("DiskSector");
    diskPhys->add(tag);
    diskPhys->add(new GeoIdentifierTag(getPhiId() ) );
    diskPhys->add(xform);
    diskPhys->add(modulePhys);

    // Now store the xform by identifier:
    Identifier id = theSensor.getID();
    DDmgr->addAlignableTransform(0,id,xform,modulePhys);
  }
  //
  // Place the supports for the disks:
  //
  GeoPixelDiskSupports pds;
  for(int ii =0; ii< pds.NCylinders(); ii++) {
    pds.SetCylinder(ii);
    Hep3Vector pos(0.,0.,pds.ZPos() );
    GeoNameTag* tag = new GeoNameTag("DiskSupport");
    GeoTransform* xform = new GeoTransform(HepTransform3D(HepRotation(),pos));
    diskPhys->add(tag);
    diskPhys->add(xform);
    diskPhys->add(pds.Build() );
  }

  return diskPhys;
}

double GeoPixelDisk::RMin() {
  return gmt_mgr->PixelECCarbonRMin("Inner");
}

double GeoPixelDisk::RMax() {
  return gmt_mgr->PixelECCarbonRMax("Outer");
}

double GeoPixelDisk::Thickness() {
  //
  // This can be calculated as the minimum thickness possible
  // as for the layers I use a number which is enough to contain
  // the inner part of the detector.
  //
  // 7-1 I switch to the minimum thickness possible as the cables are right
  // outside this volume.
  //
  //  return 10*mm;
  double tck = 2*(gmt_mgr->PixelBoardThickness()
                  +std::max(gmt_mgr->PixelHybridThickness(),gmt_mgr->PixelChipThickness()));
  tck += std::max(gmt_mgr->PixelECSiDz1(),gmt_mgr->PixelECSiDz2());
  return tck;
}


//
// This is to retrofit the convention in the pixelId
// \which is coming from atlsim...
//
int GeoPixelDisk::getPhiId() {
  int brl_ec=0;
  int phimod;
    if(gmt_mgr->isBarrel() ) brl_ec = 0;
    if(gmt_mgr->isEndcap() ) brl_ec = 2*gmt_mgr->GetSide();
    if(brl_ec == 2) {
      phimod = gmt_mgr->Phi()*2 + gmt_mgr->Eta();
    } else {
      //
      // The (eta) negative) endcap is more complicated, as there is an extra
      // rotation of the endcap as a whole around Y
      //
      phimod = 48-gmt_mgr->Phi()*2-gmt_mgr->Eta()-2;
      if (phimod == -1) phimod = 47;
    }
    return phimod;
}
