//
// This class builds one Si Crystal to be placed in one module
//
// The information on which layer to build is retrieved by the Geometry Manager
//
// Contained by: PixelModule_log
//
// Contains: nothing
//
#include "PixelGeoModel/GeoPixelSiCrystal.h"
#include "GeoModelKernel/GeoBox.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
#include "GeoModelKernel/GeoFullPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"
#include "Identifier/Identifier.h"
#include "InDetIdentifier/PixelID.h"
#include "InDetReadoutGeometry/PixelDetectorManager.h"

#include "InDetReadoutGeometry/SiDetectorElement.h"
#include "InDetReadoutGeometry/PixelModuleDesign.h"
#include "InDetReadoutGeometry/PixelDiodeMatrix.h"
#include "InDetReadoutGeometry/SiliconConditions.h"
#include "InDetReadoutGeometry/SiliconProperties.h"
#include "InDetReadoutGeometry/SiCommonItems.h"

#include <vector>

using namespace InDetDD;

GeoPixelSiCrystal::GeoPixelSiCrystal(bool isBLayer) {
  // 
  //Builds the design for this crystal
  m_isBLayer = isBLayer;
  //SiDetectorDesign::Axis etaAxis   = SiDetectorDesign::zAxis;
  //SiDetectorDesign::Axis phiAxis   = SiDetectorDesign::yAxis;
  //SiDetectorDesign::Axis depthAxis = SiDetectorDesign::xAxis;
  double thickness = gmt_mgr->PixelBoardThickness();
  int CircPerCol = gmt_mgr->DesignCircuitsPerColumn();
  int CircPerRow = gmt_mgr->DesignCircuitsPerRow();
  int CellRowPerCirc = gmt_mgr->DesignCellRowsPerCircuit(isBLayer);
  int CellColPerCirc = gmt_mgr->DesignCellColumnsPerCircuit(isBLayer);
  int DiodeRowPerCirc = gmt_mgr->DesignDiodeRowsPerCircuit(isBLayer);
  int DiodeColPerCirc = gmt_mgr->DesignDiodeColumnsPerCircuit(isBLayer);

  // Add the matrix in the same way as from AGDD... 
  //
  //double startRP = -gmt_mgr->DesignRPActiveArea()/2.;
  //double startZ =  -gmt_mgr->DesignZActiveArea()/2.;
  //double ColSize = gmt_mgr->DesignZActiveArea()/gmt_mgr->DesignCircuitsPerRow();
  double bigEtaPitch =  gmt_mgr->DesignPitchZ(isBLayer) + gmt_mgr->DesignGapZ()/2;
  double phiPitch = gmt_mgr->DesignPitchRP(isBLayer);
  double etaPitch = gmt_mgr->DesignPitchZ(isBLayer);

 
  PixelDiodeMatrix * normalCell = new PixelDiodeMatrix(phiPitch, etaPitch); 
  PixelDiodeMatrix * bigCell = new PixelDiodeMatrix(phiPitch, bigEtaPitch); 

  PixelDiodeMatrix * singleChipRow = new PixelDiodeMatrix(PixelDiodeMatrix::etaDir,
							  bigCell, 
							  normalCell, 
							  DiodeColPerCirc-2,
							  bigCell);

  PixelDiodeMatrix * singleRow = new PixelDiodeMatrix(PixelDiodeMatrix::etaDir,
						      0, singleChipRow, CircPerRow, 0);

  PixelDiodeMatrix * fullMatrix = new PixelDiodeMatrix(PixelDiodeMatrix::phiDir,
						       0, singleRow, DiodeRowPerCirc, 0);

  PixelModuleDesign *p_barrelDesign2 = new PixelModuleDesign(thickness,
							     CircPerCol,
							     CircPerRow,
							     CellColPerCirc,
							     CellRowPerCirc,
							     DiodeColPerCirc,
							     DiodeRowPerCirc,
							     fullMatrix,
							     SiliconProperties::electrons);

  // Multiple connections (ganged pixels)
  if (gmt_mgr->NumberOfEmptyRows() > 0) {
    int minRow = gmt_mgr->EmptyRows(0);
    int maxRow = minRow;
 

    for (int iConnect = 0; iConnect < gmt_mgr->NumberOfEmptyRows(); iConnect++){
      minRow = std::min(minRow, gmt_mgr->EmptyRows(iConnect));
      minRow = std::min(minRow, gmt_mgr->EmptyRowConnections(iConnect));
      maxRow = std::max(maxRow, gmt_mgr->EmptyRows(iConnect));
      maxRow = std::max(minRow, gmt_mgr->EmptyRowConnections(iConnect));
    }
  
    std::vector <int> connections(maxRow-minRow+1);
    
    // We fill them all with a one to one correspondence first.
    for (unsigned int iRow = 0; iRow < connections.size(); iRow++){
      connections[iRow] = iRow +  minRow;
    }
    
    // Now make the connections.
    for (int iConnect = 0; iConnect < gmt_mgr->NumberOfEmptyRows(); iConnect++){
      connections[gmt_mgr->EmptyRows(iConnect)-minRow] = gmt_mgr->EmptyRowConnections(iConnect);
    }
    
    //std::cout << "MinRow = " << minRow << std::endl;
    //std::cout << "MaxRow = " << maxRow << std::endl;
    //for (unsigned int iRow = 0; iRow < connections.size(); iRow++){
    //  std::cout << iRow << " " << connections[iRow] << std::endl;
    //}

    p_barrelDesign2->addMultipleRowConnection(minRow, connections);

  } else {
    // No empty rows.
    std::cout << "GeoPixelSiCrystal: Warning - No ganged pixels" << std::endl;
  }
  

  m_design = p_barrelDesign2;

}
GeoVPhysVol* GeoPixelSiCrystal::Build() {
  //
  // Dimensions
  //
  double length = gmt_mgr->PixelBoardLength();
  double thickness = gmt_mgr->PixelBoardThickness();
  double width = gmt_mgr->PixelBoardWidth();
  GeoMaterial* siMat = mat_mgr->getMaterial("std::Silicon");
  const GeoBox* siBox = new GeoBox(thickness/2.,width/2.,length/2.);
  std::string logname;
  if(m_isBLayer) logname = "siBLayLog";
  else logname = "siLog";
  GeoLogVol* theSi = new GeoLogVol(logname,siBox,siMat);
  GeoFullPhysVol* siPhys = new GeoFullPhysVol(theSi);
  //
  // Add its weight to the full detector
  //
  double weisicr = siBox->volume()*siMat->getDensity()/gram;
  //  std::cout<<"And the SiCrystal Weight is "<<weisicr<<" grams"<<std::endl;
  gmt_mgr->AddWeight(weisicr);
  gmt_mgr->AddWeightModule(weisicr);
  //
  // Add this to the list of detector elements:
  //
  int brl_ec=0;
  //
  // Build the Identifier for the silicon:
  //
  if(gmt_mgr->isBarrel() ) brl_ec = 0;
  if(gmt_mgr->isEndcap() ) brl_ec = 2*gmt_mgr->GetSide();
  PixelID *ppp = DDmgr->getIdHelper();
  Identifier idwafer;
  if(brl_ec == 0) {
    idwafer = ppp->wafer_id(brl_ec,gmt_mgr->GetLD(),gmt_mgr->Phi(),gmt_mgr->Eta());
  } else {
    int phimod;
    //
    // This is evaluating the phimod. It's a bit complicated as I have to 
    // assign the numbers corresponding to a counter-clockwise rotation
    // in the disk reference frame. For the (eta) positive endcap I just 
    // have to match the start position with the start of the rotation. 
    //
    if(brl_ec == 2) {
      phimod = gmt_mgr->Phi()*2 + gmt_mgr->Eta();
    } else {
      //
      // The (eta) negative) endcap is more complicated, as there is an extra
      // rotation of the endcap as a whole around Y
      //
      phimod = 48-gmt_mgr->Phi()*2-gmt_mgr->Eta()-2;
      if (phimod == -1) phimod = 47;
    }
      
    idwafer = ppp->wafer_id(brl_ec,gmt_mgr->GetLD(),phimod,0);
  }
  _id=idwafer;
  SiDetectorElement * element = new SiDetectorElement(idwafer, m_design, siPhys, gmt_mgr->commonItems());
  // Add conditions:  (do it here as eventually they will come from the cond DB)
  SiliconConditions conditions(gmt_mgr->Temperature(m_isBLayer), gmt_mgr->Voltage(m_isBLayer), 0.);
  element->setSiliconConditions(conditions);
  // Pass on G3ComptibleDigits option.
  element->setG3CompatibleOrientation(gmt_mgr->G3CompatibleDigits());
  // Pass the magnetic field.
  if (!gmt_mgr->commonItems()->useMagneticFieldSvc()) {
    element->setMagneticField(gmt_mgr->magneticField(m_isBLayer));
  }
  // add the element to the manager
  DDmgr->addDetectorElement(element);
  return siPhys;
}
