#include "GeoModelKernel/GeoMaterial.h"
#include "PixelGeoModel/OraclePixGeoManager.h"

// to permit access to StoreGate
#include "GaudiKernel/ISvcLocator.h"
#include "GaudiKernel/Bootstrap.h"
#include "GaudiKernel/MsgStream.h"
#include "GaudiKernel/IMessageSvc.h"
#include "AthenaKernel/getMessageSvc.h"
#include "StoreGate/StoreGateSvc.h"

// Joe's Material Manager
#include "GeoModelSvc/StoredMaterialManager.h"
#include "GeoModelSvc/IGeoModelSvc.h"
#include "GeoModelSvc/DecodeVersionKey.h"

//
// Get the pixelDD Manager from SG.
//
#include "InDetReadoutGeometry/PixelDetectorManager.h"
#include "StoreGate/StoreGate.h"
#include "StoreGate/DataHandle.h"
#include "RDBAccessSvc/IRDBAccessSvc.h"
//
// 
//
#include "CLHEP/Units/PhysicalConstants.h"

using InDetDD::PixelDetectorManager; 


using namespace std;

OraclePixGeoManager::OraclePixGeoManager()
  : m_eta(0),
    m_phi(0),
    currentLD(0),
    BarrelEndcap(0),
    _side(0),
    //JBdV new for checks
    m_nechelles(0),
    m_lepoids(0.),
    m_weiomega(0.),
    m_weicable(0.),
    m_weimod(0.),
    m_weitmt(0.),
    m_weifluid(0.),
    m_servicesOnLadder(true),  
    //
    m_services(true),  
    m_g3CompatibleDigits(false), 
    m_initialLayout(false), 
    m_dc1Geometry(false),
    m_alignable(true),
    m_magFieldFromNova(true),
    m_useMagFieldSvc(false)
{



  // Get service locator from Bootstrap
  ISvcLocator* svcLocator = Gaudi::svcLocator(); // from Bootstrap

  MsgStream log(Athena::getMessageSvc(), "PixelGeoModel");

  log << MSG::DEBUG << "Using ORACLE PIXEL GEOMETRY MANAGER" << endreq;

  StatusCode sc = svcLocator->service("DetectorStore", m_pDetStore );
  if (sc.isFailure()) log << MSG::FATAL << "Could not locate DetectorStore" << endreq;  
 
  IRDBAccessSvc *rdbSvc;
  sc = svcLocator->service("RDBAccessSvc",rdbSvc);
  if (sc.isFailure()) log << MSG::FATAL << "Could not locate RDBAccessSvc" << endreq;

  IGeoModelSvc *geoModel;
  sc = svcLocator->service ("GeoModelSvc",geoModel);
  if (sc.isFailure()) log << MSG::FATAL << "Could not locate GeoModelSvc" << endreq;

  // Get version tag and node for Pixel.
  DecodeVersionKey versionKey(geoModel, "Pixel");
  std::string detectorKey  = versionKey.tag();
  std::string detectorNode = versionKey.node();

  m_versionTag = rdbSvc->getChildTag("Pixel", versionKey.tag(), versionKey.node(), false);


/////////////////////////////////////////////////////////
//
// Gets the structures from the det store
//
/////////////////////////////////////////////////////////

  log << MSG::INFO << "Retrieving Record Sets from database ..." << endreq;
  log << MSG::INFO << "Key = " << detectorKey << " Node = " << detectorNode << endreq;

  atls = rdbSvc->getRecordset("AtlasMother",geoModel->atlasVersion(), "ATLAS");
  PixelBarrelGeneral = rdbSvc->getRecordset("PixelBarrelGeneral",     detectorKey, detectorNode);
  PixelBarrelService = rdbSvc->getRecordset("PixelBarrelService",     detectorKey, detectorNode);
  PixelCommon        = rdbSvc->getRecordset("PixelCommon",            detectorKey, detectorNode);
  PixelDisk          = rdbSvc->getRecordset("PixelDisk",              detectorKey, detectorNode);
  PixelEndcapGeneral = rdbSvc->getRecordset("PixelEndcapGeneral",     detectorKey, detectorNode);
  PixelEndcapService = rdbSvc->getRecordset("PixelEndcapService",     detectorKey, detectorNode);
  PixelLayer         = rdbSvc->getRecordset("PixelLayer",             detectorKey, detectorNode);
  PixelModule        = rdbSvc->getRecordset("PixelModule",            detectorKey, detectorNode);
  PixelStave         = rdbSvc->getRecordset("PixelStave",             detectorKey, detectorNode);

  
  std::string detectorKeyN  = "PixelLegacy-01";
  std::string detectorNodeN = "PixelLegacy";

  // These are for the design
  pxbi = rdbSvc->getRecordset("PXBI",     detectorKeyN, detectorNodeN);
  pbrn = rdbSvc->getRecordset("PBRN",     detectorKeyN, detectorNodeN); //JBdVtemporary
  pbac = rdbSvc->getRecordset("PBAC",     detectorKeyN, detectorNodeN); //JBdVtemporary
  pxei = rdbSvc->getRecordset("PXEI",     detectorKey, detectorNode);
  pdch = rdbSvc->getRecordset("PDCH",     detectorKey, detectorNode);
  pxbd = rdbSvc->getRecordset("PXBD",     detectorKey, detectorNode);
  plor = rdbSvc->getRecordset("PLOR",     detectorKey, detectorNode);
  plrn = rdbSvc->getRecordset("PLRN",     detectorKey, detectorNode);

  // These are for a detailed description of the ladders and services on ladder
  ptla = rdbSvc->getRecordset("PTLA",     detectorKeyN, detectorNodeN);
  pctr = rdbSvc->getRecordset("PCTR",     detectorKeyN, detectorNodeN);
  pftr = rdbSvc->getRecordset("PFTR",     detectorKeyN, detectorNodeN);
  pttr = rdbSvc->getRecordset("PTTR",     detectorKeyN, detectorNodeN);
  pome = rdbSvc->getRecordset("POME",     detectorKeyN, detectorNodeN);
  poti = rdbSvc->getRecordset("POTI",     detectorKeyN, detectorNodeN);
  pobi = rdbSvc->getRecordset("POBI",     detectorKeyN, detectorNodeN);
  poai = rdbSvc->getRecordset("POAI",     detectorKeyN, detectorNodeN);
  poci = rdbSvc->getRecordset("POCI",     detectorKeyN, detectorNodeN);
  posi = rdbSvc->getRecordset("POSI",     detectorKeyN, detectorNodeN);
  pccf = rdbSvc->getRecordset("PCCF",     detectorKeyN, detectorNodeN);
  pcff = rdbSvc->getRecordset("PCFF",     detectorKeyN, detectorNodeN);
  
  log << MSG::INFO << "... Record Sets retrieved." << endreq;

  // cache the number of inner frames
  
  m_barrelInFrames =  (*PixelBarrelGeneral)[0]->getInt("NFRAMEIN");
  m_endcapInFrames =  (*PixelEndcapGeneral)[0]->getInt("NFRAMEIN");
 //
 // Get the mat manager from the store:
 //
 DataHandle<StoredMaterialManager> materialManager;
 if (StatusCode::SUCCESS == m_pDetStore->retrieve(materialManager, std::string("MATERIALS"))) {
   m_pMatMgr = materialManager.ptr();
 }
 //
 // Print the version number for the barrel and endcap geometry (from NOVA)
 //  
 //cout << "Instantiating Pixel Detector" << endl;
 //cout << "Barrel Version " << this->PixelBarrelMajorVersion() << "." << this->PixelBarrelMinorVersion() << endl;
 //cout << "Endcap Version " << this->PixelEndcapMajorVersion() << "." << this->PixelEndcapMinorVersion() << endl;
}

StoredMaterialManager* OraclePixGeoManager::getMaterialManager()
{
  return m_pMatMgr;
}


/////////////////////////////////////////////////////////
//
// Setting of Layer/Disk and Barrel/EndCap
//
/////////////////////////////////////////////////////////
void OraclePixGeoManager::SetCurrentLD(int i)
{
  if(isBarrel() ) {
    if(i <= PixelBarrelNLayer()) {
      currentLD=i;
    } else {
      cout << "Layer set out of bounds, defaulting it to 0" << endl;
      currentLD = 0;
      std::cout<<" Current layer set to "<<i<<std::endl;
    }
  }
  if(isEndcap() ) {
     if (i<= PixelEndcapNDisk() ) {
       currentLD=i;
     } else {
       cout << "Disk set out of bounds, defaulting it to 0" << endl;
       currentLD = 0;
       std::cout<<" Current disk set to "<<i<<std::endl;
     }
  } 
}



void OraclePixGeoManager::SetBarrel() {
  //cout << "Setting Barrel" << endl;
  BarrelEndcap = 0;
};
void OraclePixGeoManager::SetEndcap() {
  BarrelEndcap = 1;
  //cout << "Setting Endcap" << endl;
  };
/////////////////////////////////////////////////////////
//
// Check if the current layer/disk has to be retrieved
// By default in MySQL all the three layers have to be retrieved
// If m_initialLayout is true we ignore NOVA for layer/disk 1.
// For initial layout layer=1 and disk=1 (2nd layer and disk) is missing.
//
/////////////////////////////////////////////////////////
bool OraclePixGeoManager::isLDPresent() {
  if(isBarrel()) {
    if (m_initialLayout && currentLD == 1) return false;
    std::ostringstream A;
    A << "_" << currentLD;
    if((*PixelBarrelGeneral)[0]->getInt("USELAYER"+A.str()) == 1) {
      return true;
    } else {
      return false;
    } 
  }
  if(isEndcap() ) {
    if (m_initialLayout && currentLD == 1) return false;
    std::ostringstream A;
    A << "_" << currentLD;
    if((*PixelEndcapGeneral)[0]->getInt("USEDISK"+A.str()) == 1) {
      return true;
    } else {
      return false;
    }
  }    
  return false;
}


bool OraclePixGeoManager::isBarrel() {
  if(BarrelEndcap == 0) {
    return true;
  } else {
    return false;
  }
}
bool OraclePixGeoManager::isEndcap() {
  if(BarrelEndcap == 1) {
    return true;
  } else {
    return false;
  }
  return false;
}
/////////////////////////////////////////////////////////
//
// Calculate Thickness. This is used for the materials
// which thickness is given in % of r.l.
//
/////////////////////////////////////////////////////////
double OraclePixGeoManager::CalculateThickness(double tck,string mat) {
  GeoMaterial* material =  m_pMatMgr->getMaterial(mat);
  double rl = material->getRadLength();
  return -1.*rl*tck/100.;
}
/////////////////////////////////////////////////////////
//
// Si Boards Parameters:
//
/////////////////////////////////////////////////////////
double OraclePixGeoManager::PixelBoardWidth() 
{
  //JBdV temporary if(isBarrel()) return (*PixelModule)[currentLD]->getDouble("BOARDWIDTH")*cm;
  if(isBarrel()) return (*pxbi)[currentLD]->getDouble("DYBOAR")*cm;
  if(isEndcap()) return (*PixelModule)[currentLD+3]->getDouble("BOARDWIDTH")*cm;
  return 0.;
}
double OraclePixGeoManager::PixelBoardLength() 
{
  //JBdV temporary if(isBarrel()) return (*PixelModule)[currentLD]->getDouble("BOARDLENGTH")*cm;
  if(isBarrel()) return (*pxbi)[currentLD]->getDouble("DZBOAR")*cm;
  if(isEndcap()) return (*PixelModule)[currentLD+3]->getDouble("BOARDLENGTH")*cm;
  return 0.;
}
double OraclePixGeoManager::PixelBoardThickness() 
{
  if (m_dc1Geometry && isBarrel() && (currentLD == 0)) {
    return 200*micrometer;
  }
  //JBdV temporary if(isBarrel()) return (*PixelModule)[currentLD]->getDouble("BOARDTHICK")*cm;
  if(isBarrel()) return (*pxbi)[currentLD]->getDouble("DXBOAR")*cm;
  if(isEndcap()) return (*PixelModule)[currentLD+3]->getDouble("BOARDTHICK")*cm;
  return 0.;
}
double  OraclePixGeoManager::PixelBoardActiveLen() 
{
  if(isEndcap()) return (*pxei)[currentLD]->getDouble("DRACTIVE")*cm;
  if(isBarrel()) return (*pxbi)[currentLD]->getDouble("DZELEB")*cm;
  return 0.;
}
/////////////////////////////////////////////////////////
//
// Hybrid Parameters:
//
/////////////////////////////////////////////////////////
double OraclePixGeoManager::PixelHybridWidth() 
{
  //JBdV temporary if(isBarrel()) return (*PixelModule)[currentLD]->getDouble("HYBRIDWIDTH")*cm;
  if(isBarrel()) return (*pxbi)[currentLD]->getDouble("DYHYBR")*cm;
  if(isEndcap()) return (*PixelModule)[currentLD+3]->getDouble("HYBRIDWIDTH")*cm;
  return 0.;
}
double OraclePixGeoManager::PixelHybridLength() 
{
  //JBdV temporary if(isBarrel()) return (*PixelModule)[currentLD]->getDouble("HYBRIDLENGTH")*cm;
  if(isBarrel()) return (*pxbi)[currentLD]->getDouble("DZHYBR")*cm;
  if(isEndcap()) return (*PixelModule)[currentLD+3]->getDouble("HYBRIDLENGTH")*cm;
  return 0.;
}
double OraclePixGeoManager::PixelHybridThickness() 
{
  double thick=0.;
  string mat;
  if(isBarrel()){
    // bad data name in db; here is worked around.  Should be DXHYBR
    //JBdV temporary thick =(*PixelModule)[currentLD]->getDouble("HYBRIDTHICK"); 
    if(isBarrel()) return (*pxbi)[currentLD]->getDouble("DXNYBR")*cm;
    mat = "pix::Hybrid";
  }
  if(isEndcap()){
    thick =(*PixelModule)[currentLD+3]->getDouble("HYBRIDTHICK"); 
    mat = "std::Silicon";
  }
  // if it is negative is given in % of r.l.
  if(thick > 0.) { 
    return thick*cm;
  } else {
    return CalculateThickness(thick,mat);
  }
  return 0.;
} 
/////////////////////////////////////////////////////////
//
// Chip Parameters:
//
/////////////////////////////////////////////////////////

double OraclePixGeoManager::PixelChipWidth()
{
  //JBdV temporary if(isBarrel()) return (*PixelModule)[currentLD]->getDouble("CHIPWIDTH")*cm;
  if(isBarrel()) return (*pbrn)[currentLD]->getDouble("DYCHIPS")*cm;
  if(isEndcap()) return (*PixelModule)[currentLD+3]->getDouble("CHIPWIDTH")*cm;
  return 0.;
}
double OraclePixGeoManager::PixelChipLength()
{
  //JBdV temporary if(isBarrel())return (*PixelModule)[currentLD]->getDouble("CHIPLENGTH")*cm;
  if(isBarrel()) return (*pbrn)[currentLD]->getDouble("DZCHIPS")*cm;
  if(isEndcap())return (*PixelModule)[currentLD+3]->getDouble("CHIPLENGTH")*cm;
  return 0.;
}
double OraclePixGeoManager::PixelChipGap()
{
  if(isBarrel()) return (*PixelModule)[currentLD]->getDouble("CHIPGAP")*cm;
  if(isEndcap()) return (*PixelModule)[currentLD+3]->getDouble("CHIPGAP")*cm;
  return 0.;
}
double OraclePixGeoManager::PixelChipThickness() {
  double thick=0.;
  string mat;
  if(isBarrel()){
    if(isBarrel()) return (*pbrn)[currentLD]->getDouble("DXCHIPS")*cm;
    //JBdV temporary thick =(*PixelModule)[currentLD]->getDouble("CHIPTHICK"); 
    mat = "pix::Chip";
  }
  if(isEndcap()){
    thick =(*PixelModule)[currentLD+3]->getDouble("CHIPTHICK"); 
    mat = "std::Silicon";
  } 
  // if it is negative is given in % of r.l.
  if(thick > 0.) { 
    return thick*cm;
  } else {
    return CalculateThickness(thick,mat);
  }
  return 0.;
}

/////////////////////////////////////////////////////////
//
// Pixel Disks carbon structure
//
/////////////////////////////////////////////////////////
double OraclePixGeoManager::PixelECCarbonRMin(string a) {
  if(a == "Inner") {
    return (*PixelDisk)[currentLD]->getDouble("SUP1RMIN")*cm;
  } else if (a == "Central") {
    return (*PixelDisk)[currentLD]->getDouble("SUP2RMIN")*cm;
  } else {
    return (*PixelDisk)[currentLD]->getDouble("SUP3RMIN")*cm;
  }
  return 0.;
}

double OraclePixGeoManager::PixelECCarbonRMax(string a) {
  if(a == "Inner") {
    return (*PixelDisk)[currentLD]->getDouble("SUP1RMAX")*cm;
  } else if (a == "Central") {
    return (*PixelDisk)[currentLD]->getDouble("SUP2RMAX")*cm;
  } else {
    return (*PixelDisk)[currentLD]->getDouble("SUP3RMAX")*cm;
  }
}

double OraclePixGeoManager::PixelECCarbonThickness(string a) {
  double tck;
  int imat;
  string mat[4] = {"std::Berillia","std::Carbon","pix::ECSevices","pix::Disk"} ;
  if(a == "Inner") {
    tck = (*PixelDisk)[currentLD]->getDouble("SUP1THICK");
    imat =(*PixelDisk)[currentLD]->getInt("SUP1MAT")-1;
  } else if (a == "Central") {
    tck = (*PixelDisk)[currentLD]->getDouble("SUP2THICK");
    imat =(*PixelDisk)[currentLD]->getInt("SUP2MAT")-1;
  } else {
    tck = (*PixelDisk)[currentLD]->getDouble("SUP3THICK");
    imat =(*PixelDisk)[currentLD]->getInt("SUP3MAT")-1;
  }
  if(tck>0.) {
    return tck*cm;
  } else {
    return CalculateThickness(tck,mat[imat]);
  }
  return 0.;
}
string OraclePixGeoManager::PixelECCarbonMaterial(string a) {
  string mat[4] = {"std::Berillia","std::Carbon","pix::ECSevices","pix::Disk"} ; 
  int imat;
  if(a == "Inner") {
    imat = (*PixelDisk)[currentLD]->getInt("SUP1MAT")-1;
  } else if (a == "Central") {
    imat = (*PixelDisk)[currentLD]->getInt("SUP2MAT")-1;    
  } else {
    imat = (*PixelDisk)[currentLD]->getInt("SUP3MAT")-1;
  }
  return mat[imat];
}


/////////////////////////////////////////////////////////
//
// Central Services (PBOI)
//
/////////////////////////////////////////////////////////
//
// This is complicated in the DB...
// If Rmin < 0 this means that the cylinder has to be placed only once
// So I return RMin with its sign, for further processing by the service
// methods.
// If RMax is <0 the thickness is given in % of r.l. which have to be 
// calculated by the method calculatethickness
//
// If Zmin and Zmax have DIFFERENT sign, then the thickness is given in 
// % of r.l....
//

int OraclePixGeoManager::PixelServiceNFrame(string a) {
  if(isBarrel() ) {
    if(a == "Inside") return (*PixelBarrelGeneral)[0]->getInt("NFRAMEIN");
    if(a == "Outside") return (*PixelBarrelGeneral)[0]->getInt("NFRAMEOUT");
  }
  if(isEndcap() ) {
    if(a == "Inside") return (*PixelEndcapGeneral)[0]->getInt("NFRAMEIN");
    if(a == "Outside") return (*PixelEndcapGeneral)[0]->getInt("NFRAMEOUT");   
  }
  return 0;
}

double*  OraclePixGeoManager::PixelServiceR(string a, int _n) {
  double rmin=0.;
  double rmax=0.;
  double *r = new double[2];
  if(isBarrel()) {
    if(a == "Inside") {
      rmin = (*PixelBarrelService)[_n]->getDouble("RIN");
      rmax = (*PixelBarrelService)[_n]->getDouble("ROUT");
    }
    else {
      rmin = (*PixelBarrelService)[_n+m_barrelInFrames]->getDouble("RIN");
      rmax = (*PixelBarrelService)[_n+m_barrelInFrames]->getDouble("ROUT");
    }
  }
  if(isEndcap()) {
    if(a == "Inside") {
      rmin = (*PixelEndcapService)[_n]->getDouble("RIN");
      rmax = (*PixelEndcapService)[_n]->getDouble("ROUT");
    }
    else {
      rmin = (*PixelEndcapService)[_n+m_endcapInFrames]->getDouble("RIN");
      rmax = (*PixelEndcapService)[_n+m_endcapInFrames]->getDouble("ROUT");
    }
  }
  // If this is negative this is the thickness of the cyl in % of r.l.
  r[0] = rmin*cm;
  if(rmax > 0) {
    r[1] = rmax*cm;
  } else {
    string material = PixelServiceMaterial(a,_n);
    r[1] = fabs(rmin*cm)+CalculateThickness(rmax,material);
  }
  return r;
}


double* OraclePixGeoManager::PixelServiceZ(string a,int _n) {
  double* z = new double[2];
  if(isBarrel()) {
    if(a == "Inside") {
      z[0] = (*PixelBarrelService)[_n]->getDouble("ZIN");
      z[1] = (*PixelBarrelService)[_n]->getDouble("ZOUT");
    }
    else {
      z[0] = (*PixelBarrelService)[_n+m_barrelInFrames]->getDouble("ZIN");
      z[1] = (*PixelBarrelService)[_n+m_barrelInFrames]->getDouble("ZOUT");
    }
  }
  if(isEndcap()) {
    if(a == "Inside") {
      z[0] = (*PixelEndcapService)[_n]->getDouble("ZIN");
      z[1] = (*PixelEndcapService)[_n]->getDouble("ZOUT");
    }
    else {
      z[0] = (*PixelEndcapService)[_n+m_endcapInFrames]->getDouble("ZIN");
      z[1] = (*PixelEndcapService)[_n+m_endcapInFrames]->getDouble("ZOUT");
    }
  }
  z[0] = z[0] *cm;
  if(z[0]*(z[1]) > -0.000000001) { // same sign and z[0] > 0.
    z[1] = z[1] *cm;
  } else {
    string material = PixelServiceMaterial(a,_n);
    z[1] = z[0] * (1 + CalculateThickness(z[1],material)/fabs(z[0]));
  }
  if(isEndcap() && a == "Inside" ) { // Translate to the ecnter of EndCap
    double center = ((*PixelEndcapGeneral)[0]->getDouble("ZMAX")+(*PixelEndcapGeneral)[0]->getDouble("ZMIN"))/2.*cm;
    z[0] = z[0]-center;
    z[1] = z[1]-center;
  }
  return z;
}


int OraclePixGeoManager::PixelServiceLD(string a,int _n) {
  if(isBarrel()) {
    if(a == "Inside") {return (*PixelBarrelService)[_n]->getInt("LAYERNUM")-1;}
    else {return (*PixelBarrelService)[_n+m_barrelInFrames]->getInt("LAYERNUM")-1;}
  }
  if(isEndcap()) {
    if(a == "Inside") {return (*PixelEndcapService)[_n]->getInt("LAYERNUM")-1;}
    else {return (*PixelEndcapService)[_n+m_endcapInFrames]->getInt("LAYERNUM")-1;}
  }
  return 0;
}

string OraclePixGeoManager::PixelServiceMaterial(string a,int _n) {
  int imat;
  if(isBarrel()) {
    string mat[11] = {
      "std::Berillia",
      "std::Carbon",
      "pix::Services",
      "pix::Titanium",
      "pix::MatPP11",
      "pix::MatPP12",
      "pix::MatPP13",
      "pix::MatPP14",
      "pix::MatPP15",
      "pix::MatPP16",
      "pix::MatPP17"};
    if(a == "Inside") {imat  = (*PixelBarrelService)[_n]->getInt("MATERIAL")-1;}
    else {imat  = (*PixelBarrelService)[_n+m_barrelInFrames]->getInt("MATERIAL")-1;}
    return mat[imat];
  }
  if(isEndcap()) {
    string mat[4] = {"std::Berillia","std::Carbon","pix::ECServices","pix::Disk"};
    if(a == "Inside") {imat  = (*PixelEndcapService)[_n]->getInt("MATERIAL")-1;}
    else {imat =(*PixelEndcapService)[_n+m_endcapInFrames]->getInt("MATERIAL")-1;}
    return mat[imat];
  }
  return "None";
}
bool OraclePixGeoManager::DoServices() {
  return m_services;
}
bool OraclePixGeoManager::DoServicesOnLadder() {
  return m_servicesOnLadder;
}

bool OraclePixGeoManager::G3CompatibleDigits() const {
  return m_g3CompatibleDigits;
}

bool OraclePixGeoManager::InitialLayout() const {
  return m_initialLayout;
}

bool OraclePixGeoManager::DC1Geometry() const {
  return m_dc1Geometry;
}

bool OraclePixGeoManager::Alignable() const {
  return m_alignable;
}


PixelDetectorManager* OraclePixGeoManager::GetPixelDDManager() {
  if(m_pDDmgr == NULL) {
    //
    // retrieve the pointer to the DD manager
    //
    StatusCode sc = m_pDetStore->retrieve(m_pDDmgr);  
    if (sc.isFailure()) {
      std::cout << "Cannot retrieve PixelDetectorManager" << std::endl;
    } 
  }
  return m_pDDmgr;
}  


void  OraclePixGeoManager::SetDetElementName(std::string name) {
  m_elementsObjectName = name;
}

std::string  OraclePixGeoManager::GetDetElementName() {
  return m_elementsObjectName;
}

void  OraclePixGeoManager::SetDesignName(std::string name) {
  m_designsObjectName = name;
}

std::string  OraclePixGeoManager::GetDesignName() {
  return m_designsObjectName;
}

void  OraclePixGeoManager::SetBarrelModuleName(std::string name){
  m_barrelDesignName = name;
}

std::string  OraclePixGeoManager::GetBarrelModuleName(){
  return m_barrelDesignName;
}

void  OraclePixGeoManager::SetEndcapModuleName(std::string name) {
  m_endcapDesignName = name;
}

std::string  OraclePixGeoManager::GetEndcapModuleName() {
  return m_endcapDesignName;
}

void  OraclePixGeoManager::SetLayer0ModuleName(std::string name) {
  m_blayerDesignName = name;
}

std::string  OraclePixGeoManager::GetLayer0ModuleName() {
  return m_blayerDesignName;
}


double OraclePixGeoManager::PixelLadderThickness() 
{
  //JBdV temporary double tck = (*PixelStave)[0]->getDouble("SUPPORTTHICK")
  double tck = (*pxbi)[0]->getDouble("DXLADDER");
  if( tck > 0.) {
    return tck*cm;
  } else {
    return CalculateThickness(tck,"pix::Ladder");
  }
}

double OraclePixGeoManager::PixelECCablesThickness() 
{
  double tck =  (*PixelDisk)[currentLD]->getDouble("CABLETHICK");
  if( tck > 0.) {
    return tck*cm;
  } else {
    return CalculateThickness(tck,"pix::ECCables");
  }
}

int OraclePixGeoManager::DesignCellRowsPerCircuit(bool isBLayer)
{
  double ActiveArea = DesignRPActiveArea();
  double GapRP = DesignGapRP();
  double pitchRP = DesignPitchRP(isBLayer);
  return static_cast<int>((ActiveArea - GapRP)/pitchRP + 0.5);
}

int OraclePixGeoManager::DesignCellColumnsPerCircuit(bool isBLayer)
{
  double ActiveArea = DesignZActiveArea();
  double GapZ = DesignGapZ();
  double pitchZ = DesignPitchZ(isBLayer); 
  int CPR = DesignCircuitsPerRow();
  return static_cast<int>((ActiveArea - GapZ*CPR)/(CPR*pitchZ) + 0.5);      
}

int OraclePixGeoManager::DesignDiodeRowsPerCircuit(bool isBLayer)
{
  double ActiveArea = DesignRPActiveArea();
  double pitchRP = DesignPitchRP(isBLayer);
  return static_cast<int>((ActiveArea)/pitchRP + 0.5);
}

int OraclePixGeoManager::DesignDiodeColumnsPerCircuit(bool isBLayer)
{
  double ActiveArea = DesignZActiveArea();
  double GapZ = DesignGapZ();
  double pitchZ = DesignPitchZ(isBLayer) ;
  int CPR = DesignCircuitsPerRow();
  return static_cast<int>((ActiveArea - GapZ*CPR)/(CPR*pitchZ) + 0.5);
}

  // Ganged Pixels
int OraclePixGeoManager::NumberOfEmptyRows()
{
  return static_cast<int>((*pdch)[0]->getInt("NYCONNEC"));
}

int OraclePixGeoManager::EmptyRows(int index)
{
  std::ostringstream A;
  A << "_" << index;
 return static_cast<int>((*pdch)[0]->getInt("JYEMPTY"+A.str())) - 1;
}  

int OraclePixGeoManager::EmptyRowConnections(int index)
{
  std::ostringstream A;
  A << "_" << index;
  return static_cast<int>((*pdch)[0]->getInt("JYCONNEC"+A.str())) - 1;
}

double OraclePixGeoManager::Voltage(bool isBLayer){
  // override B-layer voltage for DC1 geometry by 
  // value in old DB (approx ratio of thicknesses (200/250 = 0.8)
  // 97.1*0.8 = 77.68. In Nova its 77.7.
  if (isBLayer && m_dc1Geometry) return 77.7*volt; 
  if(isBLayer) { return (*plor)[0]->getDouble("VOLTAGE")*volt;}
  return (*plor)[1]->getDouble("VOLTAGE")*volt;
}

double OraclePixGeoManager::Temperature(bool isBLayer){
  if(isBLayer) { return (*plor)[0]->getDouble("TEMPC")*kelvin+STP_Temperature;}
  return (*plor)[1]->getDouble("TEMPC")*kelvin+STP_Temperature;
}

const HepVector3D & 
OraclePixGeoManager::magneticField(bool isBLayer) const
{
  if (m_magFieldFromNova) {
    if(isBLayer) { 
      m_magField = HepVector3D(0, 0, (*plrn)[0]->getDouble("BFIELD") * tesla);
    } else {
      m_magField = HepVector3D(0, 0, (*plrn)[1]->getDouble("BFIELD") * tesla);
    }
  }
  return m_magField;
}

void 
OraclePixGeoManager::setMagneticField(const HepVector3D & field)
{
  m_magField = field;
  m_magFieldFromNova = false;
  m_useMagFieldSvc = false;
}

bool
OraclePixGeoManager::useMagneticFieldSvc() const
{
  return m_useMagFieldSvc;
}

void 
OraclePixGeoManager::setUseMagneticFieldSvc(bool flag)
{
  m_useMagFieldSvc = flag;
}


InDetDD::SiCommonItems * 
OraclePixGeoManager::commonItems() const
{
  return m_commonItems;
}


void 
OraclePixGeoManager::setCommonItems(InDetDD::SiCommonItems * commonItems)
{
  m_commonItems = commonItems;
}

std::string OraclePixGeoManager::PixelGetMat(int i) {
  if (i == 1) return "pix::MatOmega";
  if (i == 2) return "pix::MatGlue";
  if (i == 3) return "pix::MatAlTube";
  if (i == 4) return "pix::MatPigtail";
  if (i == 5) return "pix::MatConn";
  if (i == 6) return "pix::MatT0";
  if (i == 7) return "pix::MatAlTubeFix";
  if (i == 8) return "pix::MatTMT";
  if (i == 9) return "pix::MatCap1";
  if (i == 10) return "pix::MatCap2";
  std::cout <<"Unkown material type ! return matOmega" << std::endl;
  return "pix::MatOmega";
}
