
#include "PixelGeoModel/PixelDetectorTool.h"
#include "PixelGeoModel/PixelDetectorFactory.h" 
#include "PixelGeoModel/PixelDetectorFactoryDC2.h" 
#include "PixelGeoModel/PixelGeometryManager.h" 
#include "PixelGeoModel/PixelSwitches.h" 
#include "InDetReadoutGeometry/PixelDetectorManager.h" 

#include "GeoModelSvc/GeoModelExperiment.h"
#include "GaudiKernel/IService.h"
#include "GaudiKernel/ISvcLocator.h"
#include "GaudiKernel/MsgStream.h"
#include "StoreGate/StoreGateSvc.h"
#include "GeoModelSvc/IGeoModelSvc.h"
#include "GeoModelSvc/DecodeVersionKey.h"

#include "RDBAccessSvc/IRDBAccessSvc.h"
#include "RDBAccessSvc/IRDBRecord.h"
#include "RDBAccessSvc/IRDBRecordset.h"

using InDetDD::PixelDetectorManager; 
using InDetDD::SiDetectorManager; 

/**
 ** Constructor(s)
 **/
PixelDetectorTool::PixelDetectorTool( const std::string& type, const std::string& name, const IInterface* parent )
  : GeoModelTool( type, name, parent ),
    m_detectorName("PixelDetector")
{
  declareProperty("Services",m_services=true);
  declareProperty("ServicesOnLadder",m_servicesOnLadder=true); ///JBdV
  declareProperty("G3DigitsCompatible",m_g3CompatibleDigits=false);
  declareProperty("Alignable", m_alignable=false);
  declareProperty("UseMagFieldSvc", m_useMagFieldSvc=false);
  declareProperty("DC1Geometry",m_dc1Geometry=false); 
  declareProperty("InitialLayout",m_initialLayout=false);
  declareProperty("DevVersion", m_devVersion=true);

}
/**
 ** Destructor
 **/
PixelDetectorTool::~PixelDetectorTool()
{
  // This will need to be modified once we register the Pixel DetectorNode in
  // the Transient Detector Store
  if ( 0 != m_detector ) {
    delete m_detector;
    m_detector = 0;
  }
}

/**
 ** Create the Detector Node corresponding to this tool
 **/
StatusCode PixelDetectorTool::create( StoreGateSvc* detStore )
{ 


  MsgStream log(msgSvc(), name());  

  StatusCode result = StatusCode::SUCCESS;


  if (m_devVersion) {
    log << MSG::WARNING << "You are using a development version. There are no guarantees of stability"
	<< endreq;
  }
   

  // Get the detector configuration.
  IGeoModelSvc *geoModel;
  StatusCode sc = service ("GeoModelSvc",geoModel);
  if (sc.isFailure()) {
    log << MSG::FATAL << "Could not locate GeoModelSvc" << endreq;
    return (StatusCode::FAILURE); 
  }  
 
  DecodeVersionKey versionKey(geoModel, "Pixel");

  log << MSG::INFO << "Building Pixel Detector with Version Tag: " << versionKey.tag() << " at Node: " 
      << versionKey.node() << endreq;

  IRDBAccessSvc *accessSvc;
  std::string pixelVersionTag;

  sc = service("RDBAccessSvc",accessSvc);
  if (sc.isFailure()) {
    log << MSG::FATAL << "Could not locate RDBAccessSvc" << endreq;
    return (StatusCode::FAILURE); 
  }  

  // Print the SCT version tag:
  pixelVersionTag = accessSvc->getChildTag("Pixel", versionKey.tag(), versionKey.node(), false);
  log << MSG::INFO << "Pixel Version: " << pixelVersionTag << endreq;
  
  
  //JBdV do not take the Services property from DB !
  bool m_servicesb = m_services;

  // Check if version is empty. If so, then the SCT cannot be built. This may or may not be intentional. We
  // just issue an INFO message. 
  if (pixelVersionTag.empty()) { 
    log << MSG::INFO << "No Pixel Version. Pixel Detector will not be built." << endreq;
     
  } else {

    // Unless we are using custom pixel, the switch positions are going to
    // come from the database:
    
    if (versionKey.custom()) {

      log << MSG::WARNING << "PixelDetectorTool:  Detector Information coming from a custom configuration!!" << endreq;
 
    } else {

      log << MSG::DEBUG << "PixelDetectorTool:  Detector Information coming from the database and job options IGNORED." << endreq;
      service("RDBAccessSvc",accessSvc);
      log << MSG::DEBUG << "Keys for Pixel Switches are "  << versionKey.tag()  << "  " << versionKey.node() << endreq;
      const IRDBRecordset *switchSet = accessSvc->getRecordset("PixelSwitches", versionKey.tag(), versionKey.node());
      const IRDBRecord    *switcheTable   = (*switchSet)[0];
      
      m_services           = switcheTable->getInt("BUILDSERVICES");
      m_g3CompatibleDigits = switcheTable->getInt("G3DIGITSCOMPATIBLE");
      //m_alignable          = switcheTable->getInt("ALIGNABLE");
      m_useMagFieldSvc     = switcheTable->getInt("USEMAGFIELDSVC");
      m_dc1Geometry        = switcheTable->getInt("DC1GEOMETRY");
      m_initialLayout      = switcheTable->getInt("INITIALLAYOUT");
    }
  
    log << MSG::DEBUG  << "Creating the Pixel " << endreq;
    log << MSG::DEBUG  << "Pixel Geometry Options:" << endreq;
    log << MSG::DEBUG  << "  Services           = " << (m_services ? "true" : "false") << endreq;
    log << MSG::DEBUG  << "  G3DigitsCompatible = " << (m_g3CompatibleDigits ? "true" : "false") <<endreq;
    log << MSG::DEBUG  << "  Alignable          = " << (m_alignable ? "true" : "false") <<endreq;
    log << MSG::DEBUG  << "  UseMagFieldSvc     = " << (m_useMagFieldSvc ? "true" : "false") <<endreq;
    log << MSG::DEBUG  << "  DC1Geometry        = " << (m_dc1Geometry ? "true" : "false") <<endreq;
    log << MSG::DEBUG  << "  InitialLayout      = " << (m_initialLayout ? "true" : "false") <<endreq;


    //
    // Initialize the geometry manager
    //

    // Initialize switches
    PixelSwitches switches;
    
    switches.setServices(m_services);
    switches.setG3CompatibleDigits(m_g3CompatibleDigits);
    switches.setDC1Geometry(m_dc1Geometry);
    switches.setAlignable(m_alignable);
    switches.setInitialLayout(m_initialLayout);

    //JBdV
    switches.setServicesOnLadder(m_servicesOnLadder);
    switches.setServices(m_servicesb); //Overwrite there for the time being.

    
    if (m_g3CompatibleDigits && m_useMagFieldSvc) {
      log << MSG::WARNING << " G3DigitsCompatible is true and so UseMagFieldSvc will be set to false." << endreq;
      m_useMagFieldSvc = false;
    }
    
    switches.setUseMagneticFieldSvc(m_useMagFieldSvc);

    
    // 
    // Locate the top level experiment node 
    // 
    DataHandle<GeoModelExperiment> theExpt; 
    if (StatusCode::SUCCESS != detStore->retrieve( theExpt, "ATLAS" )) { 
      log << MSG::ERROR 
	  << "Could not find GeoModelExperiment ATLAS" 
	  << endreq; 
      return (StatusCode::FAILURE); 
    } 
    
    GeoPhysVol *world=&*theExpt->getPhysVol();
    const PixelDetectorManager* manager = 0;
 
    if (!m_devVersion) {
      // DC1/DC2 version
      PixelDetectorFactoryDC2 thePixel(detStore, switches);
      thePixel.create(world);      
      manager  = thePixel.getDetectorManager();
  
    } else {
      //
      // DEVELOPMENT VERSIONS
      //
      PixelDetectorFactory thePixel(detStore, switches);
      thePixel.create(world);      
      manager  = thePixel.getDetectorManager();
    }

    // Register the manager to the Det Store    
    if (StatusCode::FAILURE == detStore->record(manager, manager->getName()) ) {
      log << MSG::ERROR << "Could not register Pixel detector manager" << endreq;
      return( StatusCode::FAILURE );
    }
    // Add the manager to the experiment 
    theExpt->addManager(manager);
    
    // Symlink the manager
    const SiDetectorManager * siDetManager = manager;
    if (StatusCode::FAILURE == detStore->symLink(manager, siDetManager) ) { 
      log << MSG::ERROR << "Could not make link between PixelDetectorManager and SiDetectorManager" << endreq;
      return( StatusCode::FAILURE );
    }
  }

  return result;
}
  
  
